#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------------------------
#
# Copyright (c) 2022 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a Python plug-in for GIMP. It is a Python implementation of the amazing Bevel Reflect Logo script (version 1.6) by GnuTux (2013) in order to apply the MLBevelLogo2 plug-in to all the character layers in a custom font file.
# It simply applies the work principally of GnuTux (2013) and MareroQ (2020) in batch mode.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Custom Font Bevel Reflect V 1.09 Tabbed...

# version 1.00
#     The filter will complain if it cannot locate MLBevelLogo2 and end
#     Most of the options available from the user dialog mirror the options available in the original script dialog by GnuTux
#     with the following additional options:
#        Option to trim pixels from the character should the user need to tidy up the character edges
#        Option to desaturate output from BRL to apply the generated highlights to an existing custom font
#        Option to generate different emaps for different character groups
#        Option to save any generated emaps as a new image

# version 1.01
#		Fixed trim option - was trimming the wrong layer
# version 1.02
#        Blind attempt to resolve calling error on SamJ portable
# version 1.03
#        Resolved calling error on SamJ portable
# version 1.04
#        Random choice of emap extended to all layers in file - not just the first five
# version 1.05
#        Added an extra randomization option - without clumping ie should no longer get multiple adjacent copies of the same emap (just two together occasionally)
# version 1.06
#        Added an option to force the position of layers to the image origin
#            When this is not enabled all custom font layers are made visible
#		 Auto-crops loaded emaps to remove excess transparent borders that can affect quality of output
#        Fixed missing high-value unicode characters
# version 1.07
#        Apply selective gaussian blur to remove low contrast artifacts
# version 1.08
#        Removed texture sizes from dialog to reduce dialog size (texture sizes remain at default values)
# version 1.08 Tabbed
#        Added Gimpui3 and tabs thanks to the work of Arakne and MrQ to save display space for the dialog
# version 1.09 Tabbed
#        Added single character preview to new image option
#        Added force no emap option so that user can revert to using generated emaps after using emaps from file
# version 1.10 Tabbed
#        Fixed error in contour curves splines (thanks to MrQ for reporting it)
#		 Reverted to using deprecated pdb for gimp splines because of unsatisfactory results
#        Adjusted outline contours thickness to suit custom font use
#        Added option to add outlines to characters
#        Various dialog option wordings changed
# version 1.10-P MrQ added a Preview
#        by fu_preview (modificed gimpfu) by Claude Lion [aka cli345] http://gimpchat.com/viewtopic.php?f=9&t=19420
#        & gimfu3 with tabs by J.F. Garcia [arakne]  https://www.arakne.es/en/gimp-plugin-follow-path-map-values-on-the-y-coordinate/

#####################################################################################################################################################
############################################# START fu_preview 0.1.1 by Claude Lion $ J.F. Garcia ###################################################
#####################################################################################################################################################
import os, sys,time
import string as _string
import math
import gimp
import gimpcolor
import colorsys
import re
from gimpenums import *
pdb = gimp.pdb
import random
class error(RuntimeError): pass
class CancelError(RuntimeError): pass

PF_INT8        = PDB_INT8
PF_INT16       = PDB_INT16
PF_INT32       = PDB_INT32
PF_INT         = PF_INT32
PF_FLOAT       = PDB_FLOAT
PF_STRING      = PDB_STRING
PF_VALUE       = PF_STRING
PF_COLOR       = PDB_COLOR
PF_COLOUR      = PF_COLOR
PF_ITEM        = PDB_ITEM
PF_DISPLAY     = PDB_DISPLAY
PF_IMAGE       = PDB_IMAGE
PF_LAYER       = PDB_LAYER
PF_CHANNEL     = PDB_CHANNEL
PF_DRAWABLE    = PDB_DRAWABLE
PF_VECTORS     = PDB_VECTORS

PF_TOGGLE      = 1000
PF_BOOL        = PF_TOGGLE
PF_SLIDER      = 1001
PF_SPINNER     = 1002
PF_ADJUSTMENT  = PF_SPINNER

PF_FONT        = 1003
PF_FILE        = 1004
PF_BRUSH       = 1005
PF_PATTERN     = 1006
PF_GRADIENT    = 1007
PF_RADIO       = 1008
PF_TEXT        = 1009
PF_PALETTE     = 1010
PF_FILENAME    = 1011
PF_DIRNAME     = 1012
PF_OPTION      = 1013
PF_TAB         = 1014
PF_DRAW        = 1015
PF_TITLE       = 1016

_type_mapping = {
    PF_INT8        : PDB_INT8,
    PF_INT16       : PDB_INT16,
    PF_INT32       : PDB_INT32,
    PF_FLOAT       : PDB_FLOAT,
    PF_STRING      : PDB_STRING,
    PF_COLOR       : PDB_COLOR,
    PF_ITEM        : PDB_ITEM,
    PF_DISPLAY     : PDB_DISPLAY,
    PF_IMAGE       : PDB_IMAGE,
    PF_LAYER       : PDB_LAYER,
    PF_CHANNEL     : PDB_CHANNEL,
    PF_DRAWABLE    : PDB_DRAWABLE,
    PF_VECTORS     : PDB_VECTORS,

    PF_TOGGLE      : PDB_INT32,
    PF_SLIDER      : PDB_FLOAT,
    PF_SPINNER     : PDB_INT32,

    PF_FONT        : PDB_STRING,
    PF_FILE        : PDB_STRING,
    PF_BRUSH       : PDB_STRING,
    PF_PATTERN     : PDB_STRING,
    PF_GRADIENT    : PDB_STRING,
    PF_RADIO       : PDB_STRING,
    PF_TEXT        : PDB_STRING,
    PF_PALETTE     : PDB_STRING,
    PF_FILENAME    : PDB_STRING,
    PF_DIRNAME     : PDB_STRING,
    PF_OPTION      : PDB_INT32,
    PF_TAB         : PDB_STRING,
    PF_DRAW        : PDB_INT32,
    PF_TITLE       : PDB_STRING,
}

_obj_mapping = {
    PF_INT8        : int,
    PF_INT16       : int,
    PF_INT32       : int,
    PF_FLOAT       : float,
    PF_STRING      : str,
    PF_COLOR       : gimpcolor.RGB,
    PF_ITEM        : int,
    PF_DISPLAY     : gimp.Display,
    PF_IMAGE       : gimp.Image,
    PF_LAYER       : gimp.Layer,
    PF_CHANNEL     : gimp.Channel,
    PF_DRAWABLE    : gimp.Drawable,
    PF_VECTORS     : gimp.Vectors,

    PF_TOGGLE      : bool,
    PF_SLIDER      : float,
    PF_SPINNER     : int,

    PF_FONT        : str,
    PF_FILE        : str,
    PF_BRUSH       : str,
    PF_PATTERN     : str,
    PF_GRADIENT    : str,
    PF_RADIO       : str,
    PF_TEXT        : str,
    PF_PALETTE     : str,
    PF_FILENAME    : str,
    PF_DIRNAME     : str,
    PF_OPTION      : int,
    PF_TAB         : str,
    PF_DRAW        : int,
    PF_TITLE       : str,
}

_registered_plugins_ = {}


def register(proc_name, blurb, help, author, copyright, date, label,
             imagetypes, params, results, function,
             menu=None, domain=None, on_query=None, on_run=None):
    """This is called to register a new plug-in."""

    # First perform some sanity checks on the data
    def letterCheck(str):
        allowed = _string.letters + _string.digits + "_" + "-"
        for ch in str:
            if not ch in allowed:
                return 0
        else:
            return 1

    if not letterCheck(proc_name):
        raise error, "procedure name contains illegal characters"

    for ent in params:
        if len(ent) < 4:
            raise error, ("parameter definition must contain at least 4 "
                          "elements (%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != int:
            raise error, "parameter types must be integers"

        if not letterCheck(ent[1]):
            raise error, "parameter name contains illegal characters"

    for ent in results:
        if len(ent) < 3:
            raise error, ("result definition must contain at least 3 elements "
                          "(%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != type(42):
            raise error, "result types must be integers"

        if not letterCheck(ent[1]):
            raise error, "result name contains illegal characters"

    plugin_type = PLUGIN

    if (not proc_name.startswith("python-") and
        not proc_name.startswith("python_") and
        not proc_name.startswith("extension-") and
        not proc_name.startswith("extension_") and
        not proc_name.startswith("plug-in-") and
        not proc_name.startswith("plug_in_") and
        not proc_name.startswith("file-") and
        not proc_name.startswith("file_")):
           proc_name = "python-fu-" + proc_name

    # if menu is not given, derive it from label
    need_compat_params = False
    if menu is None and label:
        fields = label.split("/")
        if fields:
            label = fields.pop()
            menu = "/".join(fields)
            need_compat_params = True

            import warnings
            message = ("%s: passing the full menu path for the menu label is "
                       "deprecated, use the 'menu' parameter instead"
                       % (proc_name))
            warnings.warn(message, DeprecationWarning, 3)

        if need_compat_params and plugin_type == PLUGIN:
            file_params = [(PDB_STRING, "filename", "The name of the file", ""),
                           (PDB_STRING, "raw-filename", "The name of the file", "")]

            if menu is None:
                pass
            elif menu.startswith("<Load>"):
                params[0:0] = file_params
            elif menu.startswith("<Image>") or menu.startswith("<Save>"):
                params.insert(0, (PDB_IMAGE, "image", "Input image", None))
                params.insert(1, (PDB_DRAWABLE, "drawable", "Input drawable", None))
                if menu.startswith("<Save>"):
                    params[2:2] = file_params

    _registered_plugins_[proc_name] = (blurb, help, author, copyright,
                                       date, label, imagetypes,
                                       plugin_type, params, results,
                                       function, menu, domain,
                                       on_query, on_run)

def _query():
    for plugin in _registered_plugins_.keys():
        (blurb, help, author, copyright, date,
         label, imagetypes, plugin_type,
         params, results, function, menu, domain,
         on_query, on_run) = _registered_plugins_[plugin]

        def make_params(params):
            return [(_type_mapping[x[0]],
                     x[1],
                     _string.replace(x[2], "_", "")) for x in params]

        params = make_params(params)
        # add the run mode argument ...
        params.insert(0, (PDB_INT32, "run-mode",
                                     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"))

        results = make_params(results)

        if domain:
            try:
                (domain, locale_dir) = domain
                gimp.domain_register(domain, locale_dir)
            except ValueError:
                gimp.domain_register(domain)

        gimp.install_procedure(plugin, blurb, help, author, copyright,
                               date, label, imagetypes, plugin_type,
                               params, results)

        if menu:
            gimp.menu_register(plugin, menu)
        if on_query:
            on_query()

def getXtra(xtra, valName, defValue):
    p = defValue
    if xtra.get(valName):
        p = xtra.get(valName)
    return p

def _get_defaults(proc_name):
    import gimpshelf

    (blurb, help, author, copyright, date,
     label, imagetypes, plugin_type,
     params, results, function, menu, domain,
     on_query, on_run) = _registered_plugins_[proc_name]

    key = "python-fu-save--" + proc_name

    if gimpshelf.shelf.has_key(key):
        return gimpshelf.shelf[key]
    else:
        # return the default values
        return [x[3] for x in params]

def _set_defaults(proc_name, defaults):
    import gimpshelf

    key = "python-fu-save--" + proc_name
    gimpshelf.shelf[key] = defaults

def _interact(proc_name, start_params):
    (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run) = _registered_plugins_[proc_name]

    def run_script(run_params):
        params = start_params + tuple(run_params)
        _set_defaults(proc_name, params)
        return apply(function, params)

    params = params[len(start_params):]

    # short circuit for no parameters ...
    if len(params) == 0:
         return run_script([])

    import pygtk
    pygtk.require('2.0')

    import gimpui, gtk
#    import pango
    gimpui.gimp_ui_init () # add MrQ
    defaults = _get_defaults(proc_name)
    defaults = defaults[len(start_params):]

    class EntryValueError(Exception):
        pass

    def warning_dialog(parent, primary, secondary=None):
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE, primary)
        if secondary:
            dlg.format_secondary_text(secondary)
        dlg.run()
        dlg.destroy()

    def error_dialog(parent, proc_name):
        import sys, traceback

        exc_str = exc_only_str = _("Missing exception information")
        try:
            etype, value, tb = sys.exc_info()
            exc_str = "".join(traceback.format_exception(etype, value, tb))
            exc_only_str = "".join(traceback.format_exception_only(etype, value))
        finally:
            etype = value = tb = None

        title = _("An error occurred running %s") % proc_name
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, title)
        dlg.format_secondary_text(exc_only_str)

        alignment = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
        alignment.set_padding(0, 0, 12, 12)
        dlg.vbox.pack_start(alignment)
        alignment.show()

        expander = gtk.Expander(_("_More Information"));
        expander.set_use_underline(True)
        expander.set_spacing(6)
        alignment.add(expander)
        expander.show()

        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.set_size_request(-1, 200)
        expander.add(scrolled)
        scrolled.show()

        label = gtk.Label(exc_str)
        label.set_alignment(0.0, 0.0)
        label.set_padding(6, 6)
        label.set_selectable(True)
        scrolled.add_with_viewport(label)
        label.show()

        def response(widget, id):
            widget.destroy()

        dlg.connect("response", response)
        dlg.set_resizable(True)
        dlg.show()

    # define a mapping of param types to edit objects ...
    class StringEntry(gtk.Entry):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.Entry.__init__(self)
            self.set_text(str(default))
            self.set_activates_default(True)
            #~ self.connect('paste-clipboard', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('inserted-text', lambda _1, _2, _3, _4: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('deleted-text', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate()) # modif C.L

        def get_value(self):
            return self.get_text()


    class LabelTitle(gtk.Label):
        #def __init__ (self, default="", **kwargs): # modif C.L. ???
        def __init__(self, default="", align = (0.0, 0.0)):		
            gtk.Label.__init__(self)
            self.set_label(str(default))
            self.set_use_markup(True)
            if align != None:
                self.set_alignment(align[0],align[1])
        def get_value(self):
            return self.get_label()

    class TextEntry(gtk.ScrolledWindow):
        def __init__ (self, default="", **kwargs): # modif C.L.
            gtk.ScrolledWindow.__init__(self)
            self.set_shadow_type(gtk.SHADOW_IN)

            self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.set_size_request(100, -1)

            self.view = gtk.TextView()
            self.add(self.view)
            self.view.show()
            self.buffer = self.view.get_buffer()
            self.set_value(str(default))
            self.buffer.connect('changed', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.			

        def set_value(self, text):
            self.buffer.set_text(text)

        def get_value(self):
            return self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter())

    class IntEntry(StringEntry):
        def get_value(self):
            try:
                return int(self.get_text())
            except ValueError, e:
                raise EntryValueError, e.args

    class FloatEntry(StringEntry):
            def get_value(self):
                try:
                    return float(self.get_text())
                except ValueError, e:
                    raise EntryValueError, e.args

    def precision(step):
        # calculate a reasonable precision from a given step size
        if math.fabs(step) >= 1.0 or step == 0.0:
            digits = 0
        else:
            digits = abs(math.floor(math.log10(math.fabs(step))));
        if digits > 20:
            digits = 20
        return int(digits)

    class SliderEntry(gtk.HScale):
        # bounds is (upper, lower, step)
        def __init__(self, default=0, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            # modif C.L.
            def react(adj):
                return self.get_toplevel().la_previ.invalidate()
            #~ self.adj.connect('value-changed', lambda adj: self.get_toplevel().la_previ.invalidate())
            self.adj.connect('value-changed', react)
            # end modif C.L.
            gtk.HScale.__init__(self, self.adj)
            self.set_digits(precision(step))

        def get_value(self):
            return self.adj.value

    class SpinnerEntry(gtk.SpinButton):
        # bounds is (upper, lower, step)
        def __init__(self, default=10, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            self.adj.connect('value-changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            gtk.SpinButton.__init__(self, self.adj, step, precision(step))


    class ToggleEntry(gtk.ToggleButton):
        #~ def __init__(self, default=0):
        def __init__(self, default=0,labels=[_("DON'T USE"),_("YES")], **kwargs): # modif C.L.
            
            gtk.ToggleButton.__init__(self)
            self.labels = labels
            self.label = gtk.Label(labels[0])
            self.add(self.label)
            self.label.show()

            #~ self.connect("toggled", self.changed)
            # modif C.L.
            def react(tog):
                self.changed(tog)
                return kwargs['dialg'].la_previ.invalidate()
            self.connect("toggled", react)
            # end modif C.L.


            self.set_active(default)

        def changed(self, tog):
            if tog.get_active():
                self.label.set_text(self.labels[1])
            else:
                self.label.set_text(self.labels[0])

        def get_value(self):
            return self.get_active()

    class RadioEntry(gtk.VBox):
        def __init__(self, default=0, items=((_("YES"), 1), (_("No"), 0)), **kwargs): # modif C.L.
            gtk.VBox.__init__(self, homogeneous=False, spacing=2)

            button = None

            for (label, value) in items:
                button = gtk.RadioButton(button, label)
                self.pack_start(button)
                button.show()

                button.connect("toggled", self.changed, value)

                if value == default:
                    button.set_active(True)
                    self.active_value = value

        def changed(self, radio, value):
            if radio.get_active():
                self.active_value = value

        def get_value(self):
            return self.active_value

    class ComboEntry(gtk.ComboBox):
        def __init__(self, default=0, items=(), **kwargs): # modif C.L.
            store = gtk.ListStore(str)
            for item in items:
                store.append([item])

            gtk.ComboBox.__init__(self, model=store)

            cell = gtk.CellRendererText()
            self.pack_start(cell)
            self.set_attributes(cell, text=0)

            self.set_active(default)
            self.connect('changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.

        def get_value(self):
            return self.get_active()

    def FileSelector(default="", title=None, **kwargs): # modif C.L.
        # FIXME: should this be os.path.separator?  If not, perhaps explain why?
        if default and default.endswith("/"):
            if default == "/": default = ""
            return DirnameSelector(default)
        else:
            return FilenameSelector(default, title=title, save_mode=False)

    class FilenameSelector(gtk.HBox):
        #gimpfu.FileChooserButton
        def __init__(self, default, save_mode=True, title=None, **kwargs): # modif C.L.
            super(FilenameSelector, self).__init__()
            if not title:
                self.title = _("Python-Fu File Selection")
            else:
                self.title = title
            self.save_mode = save_mode
            box = self
            self.entry = gtk.Entry()
            image = gtk.Image()
            image.set_from_stock(gtk.STOCK_FILE, gtk.ICON_SIZE_BUTTON)
            self.button = gtk.Button()
            self.button.set_image(image)
            box.pack_start(self.entry)
            box.pack_start(self.button, expand=False)
            self.button.connect("clicked", self.pick_file)
            if default:
                self.entry.set_text(default)

        def show(self):
            super(FilenameSelector, self).show()
            self.button.show()
            self.entry.show()

        def pick_file(self, widget):
            entry = self.entry
            dialog = gtk.FileChooserDialog(
                         title=self.title,
                         action=(gtk.FILE_CHOOSER_ACTION_SAVE
                                     if self.save_mode else
                                 gtk.FILE_CHOOSER_ACTION_OPEN),
                         buttons=(gtk.STOCK_CANCEL,
                                gtk.RESPONSE_CANCEL,
                                gtk.STOCK_SAVE
                                     if self.save_mode else
                                gtk.STOCK_OPEN,
                                gtk.RESPONSE_OK)
                        )
            dialog.set_alternative_button_order ((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))
            dialog.show_all()
            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                entry.set_text(dialog.get_filename())
            dialog.destroy()

        def get_value(self):
            return self.entry.get_text()


    class DirnameSelector(gtk.FileChooserButton):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.FileChooserButton.__init__(self,
                                           _("Python-Fu Folder Selection"))
            self.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            if default:
                self.set_filename(default)

        def get_value(self):
            return self.get_filename()

    # modif C.L.
    class ColorSelector(gimpui.ColorSelector):
        def __init__(self, default=gimpcolor.RGB(1.0, 0, 0), **kwargs):
            ret = gimpui.ColorSelector.__init__(self, default)
            self.connect('color-changed', lambda _: kwargs['dialg'].la_previ.invalidate())
            return ret
    class FontSelector(gimpui.FontSelector):
        def __init__(self, default="Sans", **kwargs):
            ret = gimpui.FontSelector.__init__(self, default)
            self.connect('font-set', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PatternSelector(gimpui.PatternSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PatternSelector.__init__(self, default)
            self.connect('pattern-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class GradientSelector(gimpui.GradientSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.GradientSelector.__init__(self, default)
            self.connect('gradient-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PaletteSelector(gimpui.PaletteSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PaletteSelector.__init__(self, default)
            self.connect('palette-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    # end modif C.L.
    class DrawCanvas(gtk.DrawingArea):
        def __init__(self,default, size=(100,100)):
            gtk.DrawingArea.__init__(self)
            self.set_size_request(size[0],size[1])
        def get_value(self):
            return 0

    _edit_mapping = {
            PF_INT8        : IntEntry,
            PF_INT16       : IntEntry,
            PF_INT32       : IntEntry,
            PF_FLOAT       : FloatEntry,
            PF_STRING      : StringEntry,
            PF_COLOR       : ColorSelector, # modif C.L.
            PF_ITEM        : IntEntry,  # should handle differently ...
            PF_IMAGE       : gimpui.ImageSelector,
            PF_LAYER       : gimpui.LayerSelector,
            PF_CHANNEL     : gimpui.ChannelSelector,
            PF_DRAWABLE    : gimpui.DrawableSelector,
            PF_VECTORS     : gimpui.VectorsSelector,

            PF_TOGGLE      : ToggleEntry,
            PF_SLIDER      : SliderEntry,
            PF_SPINNER     : SpinnerEntry,
            PF_RADIO       : RadioEntry,
            PF_OPTION      : ComboEntry,

            PF_FONT        : FontSelector,
            PF_FILE        : FileSelector,
            PF_FILENAME    : FilenameSelector,
            PF_DIRNAME     : DirnameSelector,
            PF_BRUSH       : gimpui.BrushSelector,
            PF_PATTERN     : PatternSelector,  # modif C.L.
            PF_GRADIENT    : GradientSelector, # modif C.L.
            PF_PALETTE     : PaletteSelector,  # modif C.L.
            PF_TEXT        : TextEntry,
            PF_TAB         : TextEntry,
            PF_DRAW        : DrawCanvas,
            PF_TITLE       : LabelTitle,
    }
    # JFGARCIA - modified: to pass args to the on_run
    if on_run:
        on_run(start_params+tuple(params))

    dialog = gimpui.Dialog(proc_name, "python-fu", None, 0, None, proc_name, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))

    dialog.set_alternative_button_order((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))

    dialog.set_transient()

    # modif C.L.
    hboxPrincipal = gtk.HBox(spacing=10)
	
    #dialog.vbox.pack_start(hboxPrincipal)			        # modif MrQ
    swin = gtk.ScrolledWindow()                             # New line MrQ
    swin.add_with_viewport(hboxPrincipal)                   # New Line MrQ
    dialog.vbox.pack_start(swin)                            # New Line MrQ
    swin.show()                                             # New Line MrQ	

    hboxPrincipal.show()
    
    dialog.la_previ = gimpui.DrawablePreview(start_params[1])
    dialog.la_previ.set_update(False)						# disable Preview
    dialog.la_previ.set_size_request(516,512)               # New Line MrQ	
    dialog.la_previ.la_img = start_params[0]
    dialog.la_previ.la_layer = start_params[1]
    hboxPrincipal.pack_start(dialog.la_previ)

    dialog.set_default_size(980, 650)  					# New Line MrQ
    dialog.la_previ.show()
    # end modif C.L.

    vbox = gtk.VBox(False, 2)
    vbox.set_border_width(2)
    #vbox.set_size_request(-2, 0)                           # modif C.L. + MrQ
    #~ dialog.vbox.pack_start(vbox)
    hboxPrincipal.pack_start(vbox) # modif C.L.
    vbox.show()

    if blurb:
        if domain:
            try:
                (domain, locale_dir) = domain
                trans = gettext.translation(domain, locale_dir, fallback=True)
            except ValueError:
                trans = gettext.translation(domain, fallback=True)
            blurb = trans.ugettext(blurb)
        box = gimpui.HintBox(blurb)
        vbox.pack_start(box, expand=False)
        box.show()

    table = gtk.Table(1, 2, False)
    table.set_row_spacings(2)
    table.set_col_spacings(2)
    vbox.pack_start(table, expand=True)
    table.show()

    notebook = gtk.Notebook()
    tables=[]
    for i in range(len(params)):
        if params[i][0]==PF_TAB:
            vals = params[i][4]
            currPage = params[i][3]

            for n in vals:
                tbl = gtk.Table(2, 2, False)
                tbl.set_row_spacings(5)
                tbl.set_col_spacings(5)
                tbl.show()
                tables.append({'table':tbl,'rows':0, 'totCols':0})
                notebook.append_page(tbl, gtk.Label(n))
                notebook.set_current_page(currPage)

    if len(tables)>0:
        table.attach(notebook, 0,6,0,1)
        notebook.show()
    else:
        tables.append({'table':table,'rows':0})
    # modif C.L.
    def ifinvalidated(p_la_previ):
        
        # get dimension
        #===============
        x, y = p_la_previ.get_position()
        w, h = p_la_previ.get_size()
        
        # Effet
        #=========
        # has alpha
        lb_alpha = pdb.gimp_drawable_has_alpha(p_la_previ.la_layer)
        
        # img, layer temporaires
        tempImg = pdb.gimp_image_new(w, h, 0)
        #~ tempLayer = pdb.gimp_layer_new(tempImg, w, h, 0, 'temp', 100, 0)
        tempLayer = pdb.gimp_layer_new(tempImg, w, h, lb_alpha, 'temp', 100, 0)
        pdb.gimp_image_insert_layer(tempImg, tempLayer, None, 0)
        
        # copie orig dans tempo
        rgnPreviIn = p_la_previ.la_layer.get_pixel_rgn(x, y, w, h)
        rgnTemp = tempLayer.get_pixel_rgn(0, 0, w, h)
        rgnTemp[0:w, 0:h] = rgnPreviIn[x:x+w, y:y+h]
        
        # effect
        tempImg.undo_group_start()
        tempImg.disable_undo()
        
        params = []

        try:
            for wid in edit_wids:
                params.append(wid.get_value())
        except EntryValueError:
            warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
        else:
            try:
                paramsTot = (tempImg, tempLayer) + tuple(params)
                _set_defaults(proc_name, start_params + tuple(params))
                function(*paramsTot)
                
                res = tempImg.active_layer
                
                if not lb_alpha: pdb.gimp_layer_flatten(res)
                
                tempImg.undo_group_end()
                
                rgnRes = res.get_pixel_rgn(0, 0, w, h)
                
                # mise dans previ
                #================
                p_la_previ.draw_buffer(rgnRes[0:w, 0:h], int(3+lb_alpha)*w)
                
                pdb.gimp_image_delete(tempImg)
                
                #pdb.gimp_display_new(tempImg)
            except CancelError:
                pass
            except Exception as e:
                dialog.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                error_dialog(dialog, proc_name)
                raise
    # end modif C.L.
    def response(dlg, id):
        if id == gtk.RESPONSE_OK:
            dlg.set_response_sensitive(gtk.RESPONSE_OK, False)
            dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, False)
            params = []
            try:
                for wid in edit_wids:
                    params.append(wid.get_value())
            except EntryValueError:
                warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
            else:
                try:
                    dialog.res = run_script(params)
                except Exception:
                    dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                    error_dialog(dialog, proc_name)
                    raise
        dialog.la_previ.set_update(False)	# disable Preview
        gtk.main_quit()
    dialog.la_previ.connect('invalidated', ifinvalidated) # modif C.L.
    dialog.connect("response", response)
    iniX=0
    row=0
    edit_wids = []

    for i in range(len(params)):
        paramsi = params[i]
        pf_type = paramsi[0]
        name, desc = (paramsi[1], paramsi[2])
        def_val = defaults[i]

        tab, incr, rows, connect, cols = (0, 1, 1, None, 1)
        if len(params[i])>5:
            xtra = params[i][5]
            incr = int(getXtra(xtra, 'newLine', 1))
            if incr==0 : cols = 1
            cols = getXtra(xtra, 'cols'   , cols)
            iniX = getXtra(xtra, 'iniCol' , iniX)
            rows = getXtra(xtra, 'rows'   , 1)
            tab  = getXtra(xtra, 'tab'    , 0)
            connect = getXtra(xtra,'connect', None)
        else:
            iniX = 0
        tbl = tables[tab]['table']
        row = tables[tab]['rows']


        if pf_type==PF_TAB:
            wid = _edit_mapping[pf_type](None)
        else:
            if pf_type != PF_TITLE:
                label = gtk.Label(desc)
                label.set_use_underline(True)
                label.set_alignment(0.0, 0.5)
                tbl.attach(label, iniX + 1, iniX + 2, row, row + rows, xoptions = gtk.FILL)
                label.show()

            if pf_type in (PF_SPINNER, PF_SLIDER, PF_RADIO, PF_OPTION, PF_DRAW):
                wid = _edit_mapping[pf_type](def_val, params[i][4], dialg=dialog) # modif C.L.
            elif pf_type==PF_TOGGLE:
                if isinstance(paramsi[4], list):
                    wid = _edit_mapping[pf_type](def_val, paramsi[4],dialg=dialog)
                else:
                    wid = _edit_mapping[pf_type](def_val)				


            elif pf_type==PF_TITLE:
                wid = _edit_mapping[pf_type](desc)
            else:
                wid = _edit_mapping[pf_type](def_val, dialg=dialog) # modif C.L.
            if pf_type!=PF_TITLE:
                label.set_mnemonic_widget(wid)
                tbl.attach(wid, iniX + 2, iniX + 2 + cols, row, row + rows, yoptions=0)
                iniX = iniX + cols + 1
            else:
                tbl.attach(wid, iniX, iniX+cols, row, row + rows, xoptions=gtk.FILL)
            # jfGarcia: strip html tags from toolTip
            tTip = re.sub('<[^<]+?>', '', desc)
            if pf_type != PF_TEXT:
                wid.set_tooltip_text(tTip)
            else:
                # Attach tip to TextView, not to ScrolledWindow
                wid.view.set_tooltip_text(tTip)

            wid.set_name(name)
            if connect:
                try:
                    wid.connect(connect['signal'],connect['function'])
                except Exception, e:
                    pass
            wid.show()
            wid.desc = desc

        edit_wids.append(wid)

        tables[tab]['rows'] += incr
        if incr==1:
           iniX=0

    progress_vbox = gtk.VBox(False, 6)
    vbox.pack_end(progress_vbox, expand=False)
    progress_vbox.show()

    progress = gimpui.ProgressBar()
    progress_vbox.pack_start(progress)
    progress.show()
	
    dialog.show()

    gtk.main()

    if hasattr(dialog, "res"):
        res = dialog.res
        dialog.destroy()
        return res
    else:
        dialog.destroy()
        raise CancelError

def _run(proc_name, params):
    run_mode = params[0]
    func = _registered_plugins_[proc_name][10]

    if run_mode == RUN_NONINTERACTIVE:
        return apply(func, params[1:])

    script_params = _registered_plugins_[proc_name][8]

    min_args = 0
    if len(params) > 1:
        for i in range(1, len(params)):
            param_type = _obj_mapping[script_params[i - 1][0]]
            if not isinstance(params[i], param_type):
                break

        min_args = i

    if len(script_params) > min_args:
        start_params = params[:min_args + 1]

        if run_mode == RUN_WITH_LAST_VALS:
            default_params = _get_defaults(proc_name)
            params = start_params + default_params[min_args:]
        else:
            params = start_params
    else:
       run_mode = RUN_NONINTERACTIVE

    if run_mode == RUN_INTERACTIVE:
        try:
            res = _interact(proc_name, params[1:])
        except CancelError:
            return
    else:
        res = apply(func, params[1:])

    gimp.displays_flush()

    return res

def main():
    """This should be called after registering the plug-in."""
    gimp.main(None, None, _query, _run)

def fail(msg):
    """Display an error message and quit"""
    gimp.message(msg)
    raise error, msg

def N_(message):
    return message
	
	
#####################################################################################################################################################
############################################# END fu_preview 0.1.1 by Claude Lion $ J.F. Garcia #####################################################
#####################################################################################################################################################
import os, time, sys
import gettext
import math, random
t = gettext.translation("gimp20-python", gimp.locale_directory, fallback=True)
_ = t.ugettext

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)



def do_stuff(image, layer, trim_value, desaturate_option, slope_options, 
            flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, 
            diffuse_shiny, contour, blur_depth, outline_to_original, edge_smoothing, force_no_emap, emap_name, map_type, 
            keep_emap, paint_mode, blend_direction, blend_shape, light_direction, specular_highlight, 
            detail_level, blur_radius, 
            emap_to_character, keep_own_layer, to_origin, artifact_radius, artifact_delta, do_preview, preview_char, ptab):

    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	

	
	
    # check for MLBevelReflect2
    plugin = False
    num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("reflect")
    
    if "bevelreflect2" in str(plugin_real_name): plugin = True
    
    original_active_layer = image.active_layer # get active layer
	
	# set variables
    horizontal_texture_size = 1.2 # texture sizes removed from v. 1.08 dialog
    vertical_texture_size = 2.6
    bevel_gradient = pdb.gimp_context_get_gradient()
    im_height = pdb.gimp_image_height(image)
    emap_final_size = int(im_height/2)
    auto_stretch_HSV = True
    turbulent_noise = True
    keep_layers = False
    # get random number to use as turbulence seed
    random.seed()
    random.seed()
    random_number = int(time.time() )
    #pdb.gimp_message("emap_name = " + str(emap_name) )
    own_emap = False # flag for generating emap/s
    #pdb.gimp_message("emap_name = " + str(emap_name) )
	
    if force_no_emap:
        emap_name = None
		
	
    if emap_name == None or not (emap_name.endswith(".xcf") or emap_name.endswith(".XCF") or emap_name.endswith(".png") or emap_name.endswith(".PNG")):
        emap_name = "(None)"
        emap_drawable = None
    if emap_name == "": emap_name = "(None)"

    if emap_name=="(None)":
        # Prepare for generating a new emap
        own_emap = True # set flag
        # save current colours
        saved_fg_color = pdb.gimp_context_get_foreground()
        saved_bg_color = pdb.gimp_context_get_background()
	
        # get current active layer
        layer_id = pdb.gimp_image_get_active_layer(image)
        active_layer_id = pdb.gimp_image_get_active_layer(image)
	
        # Get Parent of active layer
        parent = pdb.gimp_item_get_parent(active_layer_id)
        # Save Active Layer Position
        act_lay_pos = pdb.gimp_image_get_item_position(image, active_layer_id)
        # calculate size of required emap
        im_height = pdb.gimp_image_height(image)
        emap_final_size = int(im_height)

        e_loop_max = 1
        if emap_to_character>0: e_loop_max = 5
        eloop = 0
        while eloop < e_loop_max:
            # get random number to use as turbulence seed
            random.seed()
            random.seed()
            random_number = int(time.time() )
            emap_drawable = proc_generate_emap(image, random_number, active_layer_id, emap_drawable, emap_final_size, parent, act_lay_pos, 
                    detail_level, horizontal_texture_size, vertical_texture_size, map_type, 
                    bevel_gradient, saved_fg_color, saved_bg_color, blend_direction, specular_highlight, 
                    light_direction, paint_mode, blend_shape, blur_radius) 
            if eloop == 0:
                emap_list = [emap_drawable]
            else:
                emap_list.append(emap_drawable)
            eloop += 1
        copy_of_emap_list = fn_copy_list(emap_list)
    else:
        # load emap
        emap_img = pdb.gimp_file_load(emap_name, emap_name)
        num_loaded_layers, loaded_layers = pdb.gimp_image_get_layers(emap_img)
        loaded_layer_ids = emap_img.layers
        eloop = 0
		
        e_loop_max = num_loaded_layers
		
        current_active_layer_id = pdb.gimp_image_get_active_layer(emap_img)
			
        while eloop < e_loop_max:
            emap_drawable = loaded_layer_ids[eloop]
            if eloop == 0:
                pdb.gimp_image_set_active_layer(emap_img, emap_drawable)
                pdb.plug_in_autocrop_layer(emap_img, emap_drawable)
                emap_drawable.set_offsets(0, 0)
                emap_list = [emap_drawable]
            else:
                pdb.gimp_image_set_active_layer(emap_img, emap_drawable)
                pdb.plug_in_autocrop_layer(emap_img, emap_drawable)
                emap_drawable.set_offsets(0, 0)
                emap_list.append(emap_drawable)
            eloop += 1
		
        emap_drawable = emap_list[0]
        copy_of_emap_list = fn_copy_list(emap_list)
	
    # get layer info
    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
	
    visi_loop, first_char_layer  = 0, -1
    other_layers,num_char_layers = 0,0
    apply_to_custom = False
	
    #pdb.gimp_message("preview_char = " + str(preview_char) + " do_preview = " + str(do_preview) )
	
	
    while visi_loop < num_layers:
	    # get info about layer we are working on
        layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
        layer_name = layer_name.decode('utf-8')
		
		
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
        if test_for_custom_layer:
		
            apply_to_custom = True
            if to_origin: layer_names_list[visi_loop].set_offsets(0, 0)
			
            num_char_layers += 1
            if first_char_layer < 0: 
                first_char_layer = visi_loop
                first_char_layer_name = layer_name
                #if to_origin: layer_names_list[visi_loop].set_offsets(0, 0)
		
        else:
                
            face_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            if other_layers == 0: other_layer_list = [face_name]
		
            other_layers += 1
			
        if not do_preview: pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
        visi_loop += 1
			
			
			
    if not apply_to_custom and plugin == True:	
        message_text = "No custom font character layers."
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)
		
    if not plugin:
        message_text = "BevelReflect2 plugin cannot be found.\nUnable to continue."
        apply_to_custom = False
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)
		
			
    if apply_to_custom:
        active_to_new = False
        if do_preview:
		
            # copy the preview character and paste it as a  new image
            #pdb.gimp_message("preview_char = " + str(preview_char) )
            preview_layer = pdb.gimp_image_get_layer_by_name(image, preview_char)
            success = pdb.gimp_edit_copy(preview_layer)
            preview_image = pdb.gimp_edit_paste_as_new_image(preview_layer)
            
            pdb.gimp_image_undo_group_start(preview_image)
            preview_display = pdb.gimp_display_new(preview_image)

            preview_layer_names_list = preview_image.layers

        
            first_char_layer_name = "none"
            loop = 0
	
    while loop >= 0 and apply_to_custom:
        if do_preview:
            preview_layer_id = preview_layer_names_list[loop]
            preview_layer_name = pdb.gimp_item_get_name(preview_layer_names_list[loop])
            preview_layer_name = layer_name.decode('utf-8')	######## Added to catch higher value unicodes
            
            test_for_custom_layer = True
			
            display_layer_name =  pdb.gimp_item_get_name(preview_layer_names_list[loop])
            display_layer_name = display_layer_name.decode('utf-8')	
		
            # check we have the layer id of the currently active layer
            pdb.gimp_image_set_active_layer(preview_image, preview_layer_id)
            layer_id = pdb.gimp_image_get_active_layer(preview_image)
            active_layer_id = pdb.gimp_image_get_active_layer(preview_image)		
        else:
	        # get info about layer we are working on for custom font mode
            layer_id = layer_names_list[loop]
            layer_name = pdb.gimp_item_get_name(layer_names_list[loop])
            layer_name = layer_name.decode('utf-8')	######## Added to catch higher value unicodes
		
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[loop], layer_name)
	
            # cater for unicode characters above normal range
            display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
            display_layer_name = display_layer_name.decode('utf-8')	
	
            # check we have the layer id of the currently active layer
            pdb.gimp_image_set_active_layer(image, layer_id)
            layer_id = pdb.gimp_image_get_active_layer(image)
            active_layer_id = pdb.gimp_image_get_active_layer(image)			
				
            #test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, layer_name)
		
			
        if test_for_custom_layer:
            # ensure we have the active layer and make it visible
            if do_preview:
                active_layer_id = pdb.gimp_image_get_active_layer(preview_image)
			
            else:
                active_layer_id = pdb.gimp_image_get_active_layer(image)
				
            pdb.gimp_item_set_visible(active_layer_id, True)
			
            if emap_to_character == 2 and len(emap_list)<5: # check that there are enough emaps for this feature
                emap_to_character = 0 # reset to first drawable
                
            if emap_to_character == 3 and len(emap_list)< 2: # check that there are enough emaps for this feature
                emap_to_character = 0 # reset to first drawable
			
            if emap_to_character == 0: # use only one drawable for all characters
                emap_drawable = emap_list[emap_to_character]
				
            elif emap_to_character == 1: # use random emap ensuring use of all emaps in sequence
                emap_drawable = emap_list[0] # fall-back choice of drawable
				
                random.seed()
					
                emap_drawable = random.choice(copy_of_emap_list)
				
                if len(copy_of_emap_list)>0:
                    copy_of_emap_list.remove(emap_drawable)
					
                if len(copy_of_emap_list)<1:
                    copy_of_emap_list = fn_copy_list(emap_list)
                    random.shuffle(copy_of_emap_list)
					
            elif emap_to_character == 2: # use random emap
                emap_drawable = emap_list[0] # fall-back choice of drawable
                emap_list_len = len(emap_list)
                if emap_list_len>1:
                    random.seed()
                    emap_drawable = random.choice(emap_list)
				
					
            elif emap_to_character == 3: # use character groupings
                if display_layer_name[0] in upper_case: # use first emap for upper-case 
                    emap_drawable = emap_list[0]
                elif display_layer_name[0] in lower_case: # use second emap for lower-case 
                    emap_drawable = emap_list[1]
                elif display_layer_name[0] in number_case: # use third emap for number characters
                    emap_drawable = emap_list[2]
                elif display_layer_name[0] in punctuation_case: # use fourth emap for punctuation characters
                    emap_drawable = emap_list[3]
                else:
                    emap_drawable = emap_list[4] # use fifth emap for all other characters
            elif emap_to_character == 4: # use vowel/consonant groupings
                if display_layer_name[0] in vowel_case: # use first emap for vowels 
                    emap_drawable = emap_list[0]
                elif display_layer_name[0] in consonant_case: # use second emap for consonants 
                    emap_drawable = emap_list[1]
                else:
                    emap_drawable = emap_list[2] # use fifth emap for all other characters
                    
            if do_preview:
                active_layer_id = proc_bevel_reflect(0,preview_image, active_layer_id, slope_options, 
                                                    flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, 
                                                    diffuse_shiny, contour, blur_depth, edge_smoothing, emap_name, map_type, 
                                                    paint_mode, blend_direction, blend_shape, bevel_gradient, 
                                                    light_direction, specular_highlight, detail_level, horizontal_texture_size, vertical_texture_size, 
                                                    emap_final_size, blur_radius, auto_stretch_HSV, turbulent_noise, 
                                                    keep_layers, trim_value, desaturate_option, random_number, emap_drawable, own_emap, artifact_radius, artifact_delta, outline_to_original)
			
            else:
                active_layer_id = proc_bevel_reflect(0,image, active_layer_id, slope_options, 
                                                    flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, 
                                                    diffuse_shiny, contour, blur_depth, edge_smoothing, emap_name, map_type, 
                                                    paint_mode, blend_direction, blend_shape, bevel_gradient, 
                                                    light_direction, specular_highlight, detail_level, horizontal_texture_size, vertical_texture_size, 
                                                    emap_final_size, blur_radius, auto_stretch_HSV, turbulent_noise, 
                                                    keep_layers, trim_value, desaturate_option, random_number, emap_drawable, own_emap, artifact_radius, artifact_delta, outline_to_original)
				
				
        if (num_layers - loop_ptr) > 0:
            pdb.gimp_item_set_visible(active_layer_id, False)
			
        if do_preview:
            preview_num_layers, preview_layer_ids = pdb.gimp_image_get_layers(preview_image)
            pdb.gimp_image_resize_to_layers(preview_image)
        else:
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
            pdb.gimp_image_resize_to_layers(image)
			
        loop_ptr += 1
        if do_preview:
            loop -=1
        else:
            loop = num_layers - loop_ptr
		
    #keep_own_layer = False
    if own_emap: 
        if keep_own_layer:
            pdb.gimp_selection_none(image) # Clear selection
            success = pdb.gimp_edit_copy(emap_list[0])
            if success: 
                new_image = pdb.gimp_edit_paste_as_new_image()
                if len(emap_list)>1:
                    eloop = 1
                    e_loop_maxloop = len(emap_list)
                    while eloop<len(emap_list):
                        layer_copy = pdb.gimp_layer_new_from_drawable(emap_list[eloop],new_image)
                        pdb.gimp_image_insert_layer(new_image, layer_copy, None, -1)
                        
                        eloop += 1
				
                new_display = pdb.gimp_display_new(new_image)
        eloop = 0
        if keep_emap==1:
            while eloop<len(emap_list):
            
                pdb.gimp_image_remove_layer(image, emap_list[eloop]) # and remove own emap layer/s
                eloop+=1
		
    pdb.gimp_image_resize_to_layers(image)
	
    if first_char_layer_name <> "none": # the character layer to make visible on completion
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
    else:	
        if do_preview:
            active_layer = preview_image.active_layer
            pdb.gimp_item_set_visible(active_layer, True)
            pdb.gimp_image_set_active_layer(preview_image,active_layer)
            # Set end undo point
            pdb.gimp_image_undo_group_end(preview_image)
		
        else:
            active_layer = image.active_layer
            pdb.gimp_item_set_visible(original_active_layer, True)
            pdb.gimp_image_set_active_layer(image,original_active_layer)
	
    if not to_origin: # make all custom font layers visible to finish with
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_names_list = image.layers
        visi_loop = 0
        while visi_loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
            if test_for_custom_layer:
                pdb.gimp_item_set_visible(layer_names_list[visi_loop],True) # turn off visibility
            visi_loop += 1
        
	
    pdb.gimp_displays_flush()
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()	
	
######################################################################################################################################

def fn_copy_list(new_list): 
    copy_list = new_list[:] 
    return copy_list 
	
######################################################################################################################################

def proc_apply_curves(layer, contour):
    # apply curves 
    contour = contour - 1
    # number of x,y coordinates in spline for each contour style in curves 
    # debug value following  20/08/22
    # Set value to False to use the newer non-deprecated pdb call
    deprecated = True
    if contour == 0: # outline
        #curves_list = [0.0, 0.0, 127.0, 255.0, 255.0, 0.0]
        curves_list = [0.0, 0.0, 225.0, 225.0, 255.0, 0.0]
        if deprecated:
            curves_list = [0, 0, 127, 255, 255, 0]
    elif contour == 1: # Outline (Inner Bevel) contour
        curves_list = [0.0, 0.0, 2.0, 255.0, 255.0, 0.0]
        if deprecated:
            curves_list = [0, 0, 02, 255, 255, 0]
    elif contour == 2: #  Outline (Outer Bevel)
        curves_list = [0.0, 0.0, 250.0, 255.0, 255.0, 0.0]
        if deprecated:
            curves_list = [0, 0, 250, 255, 255, 0]
    elif contour == 3: # Step Bevel
        curves_list = [0.0, 0.0, 33.0, 8.0, 64.0, 38.0, 97.0, 102.0, 128.0, 166.0, 158.0, 209.0, 191.0, 235.0, 222.0, 247.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 33, 8, 64, 38, 97, 102, 128, 166, 158, 209, 191, 235, 222, 247, 255, 255]
    elif contour == 4: # Ring (Narrow)
        curves_list = [0.0, 0.0, 16.0, 255.0, 32.0, 0.0, 125.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 16, 255, 32, 0, 125, 255]
    elif contour == 5: # Ring (Medium)
        curves_list = [0.0, 0.0, 64.0, 255.0, 127.0, 0.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 64, 255, 127, 0, 255, 255]
    elif contour == 6: # Ring (Wide)
        curves_list = [0.0, 0.0, 95.0, 255.0, 250.0, 0.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 95, 255, 250, 0, 255, 255]
    elif contour == 7: # Ring (Extra Wide)
        curves_list = [0.0, 0.0, 95.0, 255.0, 253.0, 0.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 95, 255, 253, 0, 255, 255]
    elif contour == 8: # Ring (Thick)
        curves_list = [0.0, 0.0, 33.0, 110.0, 64.0, 237.0, 97.0, 240.0, 128.0, 138.0, 158.0, 33.0, 191.0, 5.0, 222.0, 99.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 33, 110, 64, 237, 97, 240, 128, 138, 158, 33, 191, 5, 222, 99, 255, 255]
    elif contour == 9: # Double Ring
        curves_list = [0.0, 0.0, 42.0, 255.0, 43.0, 0.0, 80.0, 32.0, 151.0, 255.0, 164.0, 0.0, 223.0, 60.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 42, 255, 43, 0, 80, 32, 151, 255, 164, 0, 223, 60, 255, 255]
    elif contour == 10: # Double Ring (Narrow)
        curves_list = [0.0, 0.0, 26.0, 255.0, 27.0, 0.0, 79.0, 255.0, 108.0, 0.0, 160.0, 255.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 26, 255, 27, 0, 79, 255, 108, 0, 160, 255, 255, 255]
    elif contour == 11: # Double Ring (Wide)
        curves_list = [0.0, 0.0, 26.0, 255.0, 27.0, 0.0, 152.0, 255.0, 162.0, 0.0, 237.0, 59.0, 255.0, 255.0]
        if deprecated:
            curves_list = [0, 0, 26, 255, 27, 0, 152, 255, 162, 0, 237, 59, 255, 255]

    # replace deprecated curves-spline pdb call in original script
    if deprecated:
        pdb.gimp_curves_spline(layer, 4, len(curves_list), curves_list)
    else:
        pdb.gimp_drawable_curves_spline(layer, 4, len(curves_list), curves_list)
    

######################################################################################################################################
def proc_bevel_reflect(run_mode, image, active_layer_id, slope_options, 
                        flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, 
                        diffuse_shiny, contour, blur_depth, edge_smoothing, emap_name, map_type, 
                        paint_mode, blend_direction, blend_shape, bevel_gradient, 
                        light_direction, specular_highlight, detail_level, horizontal_texture_size, vertical_texture_size, 
                        emap_final_size, blur_radius, auto_stretch_HSV, turbulent_noise, 
                        keep_layers, trim_value, desaturate_option, random_number, emap_drawable, own_emap, artifact_radius, artifact_delta, outline_to_original):

    #### The following is basically GnuTux's code in Python
    #### active_layer_id is the logo-layer in the original script

    # get character layer name for later rename
    character_name = pdb.gimp_item_get_name(active_layer_id)
	
    saved_fg_color = pdb.gimp_context_get_foreground()
    saved_bg_color = pdb.gimp_context_get_background()
	
    # Get Parent of current character drawable
    parent = pdb.gimp_item_get_parent(active_layer_id)

    # Selection Check - if a user selection exists save it to later restore
    saved_flag = False
    sel_flag = pdb.gimp_selection_is_empty(image)
    if sel_flag == False:
        saved_selection = pdb.gimp_selection_save(image)
        pdb.gimp_selection_none(image)
        saved_flag == True

    # Save Active Layer Position
    act_lay_pos = pdb.gimp_image_get_item_position(image, active_layer_id)
	
    # Create Bump Map & Save As Layer

    # make a copy of the character layer and add the layer to the image
    position = pdb.gimp_image_get_item_position(image, active_layer_id)
	
	
    bumpmap_layer = pdb.gimp_layer_new_from_drawable(active_layer_id, image)
    pdb.gimp_image_insert_layer(image, bumpmap_layer, parent, act_lay_pos)
    pdb.gimp_item_set_name(bumpmap_layer,"BR Logo") # rename the layer
    pdb.gimp_edit_clear(bumpmap_layer)
    pdb.gimp_selection_all(image)
    pdb.gimp_context_set_foreground(white)
    pdb.gimp_edit_bucket_fill(bumpmap_layer, 0, 0, 100, 0, 0, 0, 0)
    pdb.gimp_selection_none(image)
    pdb.gimp_item_set_visible(bumpmap_layer, True)
    bumptext_layer = pdb.gimp_layer_new_from_drawable(active_layer_id, image)
    pdb.gimp_image_insert_layer(image, bumptext_layer, parent, act_lay_pos)
	
    #  Clear Away Outside of Selection
    if saved_flag == True:
        pdb.gimp_image_select_item(image, 2, saved_selection)
        pdb.gimp_selection_invert(image) # invert selection
        pdb.gimp_edit_clear(bumptext_layer)
        pdb.gimp_selection_invert(image)
        pdb.gimp_selection_none(image)
    
    # Gausian Blur For Curves
    if blur_depth > 0:
	
    # further modifications for contours which produce outlines when using BRL with custom fonts 20/08/22
	
        if contour >0 and contour<4: # ie contours which produce outlines
            blur_depth = blur_depth + 2 # lower values produce very thin outlines or no outlines which result in an empty layer
    
        #pdb.plug_in_colorify(image, bumptext_layer, white) # Color for alpha curves
        pdb.plug_in_colorify(image, bumptext_layer, (255,255,255)) # Color for alpha curves
		
        pdb.plug_in_gauss(image, bumptext_layer, blur_depth, blur_depth, 0) # Gaussien blur the shadow (radius blur)
	
        if contour>0: # Apply contour if necessary
            proc_apply_curves(bumptext_layer, contour)
            pdb.gimp_image_set_active_layer(image, bumptext_layer)
			
    #pdb.plug_in_colorify(image, bumptext_layer, black) # Ensure Drawable BLACK for ML Bevel Reflect
    pdb.plug_in_colorify(image, bumptext_layer, (0,0,0)) # Ensure Drawable BLACK for ML Bevel Reflect
    pdb.gimp_item_set_name(bumptext_layer,"Bump Text") # rename the layer
    pdb.gimp_item_set_visible(bumptext_layer, True)
	
    bumpmap_layer = pdb.gimp_image_merge_down(image, bumptext_layer, 0) # Merge Down 
    pdb.gimp_invert(bumpmap_layer) # Invert it
	

	
    pdb.gimp_image_set_active_layer(image, bumpmap_layer)
	
    #### No dialog map-type option to create just a normal map
    #if map_type == 5: # Create a Normal Map

    image_height = pdb.gimp_drawable_height(bumpmap_layer) # get height
    image_width = pdb.gimp_drawable_width(bumpmap_layer) # get width
	
					
	
    pdb.plug_in_mlbevelreflect2(image, 
                                bumpmap_layer, 
                                False, 
                                slope_options+1, 
                                flat_bevel_smoothing, 
                                flat_bevel_depth, 
                                round1_bevel_depth, 
                                round2_bevel_depth, 
                                emap_drawable, 
                                diffuse_shiny)
								
    bumpmap_layer = pdb.gimp_image_get_active_drawable(image)

    #artifact_radius, artifact_delta = 8.00, 100.00
    if own_emap==1: 
	
        pdb.plug_in_sel_gauss(image, bumpmap_layer, artifact_radius, artifact_delta) # Trying this blur here to remove low contrast artifacts
								
    #  Edge Smoothing Attempt
    #pdb.gimp_image_select_item(image, 0, active_layer_id) # selects outline of original layer
    pdb.gimp_image_select_item(image, 0, bumpmap_layer) # selects outline of bumpmap layer
	
    if edge_smoothing == 3:
        edge_smoothing = 2
        blur_radius = 4
		
    if edge_smoothing == 4:
        edge_smoothing = 2
        blur_radius = 5
		
		
    pdb.gimp_selection_shrink(image, edge_smoothing) # Shrink by edge smoothing amount
    pdb.gimp_selection_invert(image) # Invert selection
	
	
    if edge_smoothing > 0 and blur_radius > 0: 
        pdb.plug_in_gauss(image, bumpmap_layer, blur_radius, blur_radius, 1) # Blur Edge 
        
    
    pdb.gimp_edit_clear(bumpmap_layer) # remove unwanted colour from BRL
    #pdb.gimp_edit_clear(active_layer_id) # remove unwanted colour from BRL
	
    pdb.gimp_selection_invert(image) # Invert selection
    pdb.gimp_selection_none(image) # Clear selection
								
								
								
    # identify topological layer generated by the call to the script and remove it
    br_layer = pdb.gimp_image_get_layer_by_name(image, "BR Logo")
    pdb.gimp_image_set_active_layer(image, br_layer)
	
    if desaturate_option > 0: 
        pdb.gimp_drawable_desaturate(br_layer, desaturate_option-1)
        mode = 47 # grain merge
        bumpmap_layer.mode = mode # apply mode   
        # allow user to desaturate the layer generated by the BRL plug-in before applying mode effect
    else:
        if contour == 0:
            pdb.gimp_edit_clear(active_layer_id) # clear contents of original character layer
        elif (contour > 0 and contour<4):
            if outline_to_original: 
                pdb.gimp_edit_clear(active_layer_id) # clear contents of original character layer
        elif contour >= 4: 
            pdb.gimp_edit_clear(active_layer_id) # clear contents of original character layer
	
	
    # get the name of the topological map layer generated for each map type by BRL
    if map_type == 0:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Topo")
    elif map_type == 1:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Cloud")
    elif map_type == 2:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Topo")
    elif map_type == 3:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Cloud")
    elif map_type == 4:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, (character_name + " copy"))
		
	
    
    bumpmap_layer = pdb.gimp_image_merge_down(image, bumpmap_layer, 0) # merge down the copy of the character layer
    pdb.gimp_item_set_name(bumpmap_layer,character_name) # reinstate the original name of the character
    if trim_value > 0: proc_trim_character(image, bumpmap_layer, trim_value) # allow user to trim pixels from the selection outline
	

    active_layer_id = pdb.gimp_image_get_active_layer(image) # identify the new active layer

    return active_layer_id
			
#############################################################################################################################

def proc_generate_emap(image, random_number, active_layer_id, emap_drawable, emap_final_size, parent, act_lay_pos, 
                    detail_level, horizontal_texture_size, vertical_texture_size, map_type, 
                    bevel_gradient, saved_fg_color, saved_bg_color, blend_direction, specular_highlight, 
                    light_direction, paint_mode, blend_shape, blur_radius):
					
    #### This is basically GnuTux's code in Python
    emap_drawable = pdb.gimp_layer_new_from_drawable(active_layer_id, image) # New Layer for Emap
    pdb.gimp_image_insert_layer(image, emap_drawable, parent, act_lay_pos + 1) # Add Layer
    pdb.gimp_image_set_active_layer(image, emap_drawable)
    pdb.gimp_layer_resize(emap_drawable, emap_final_size + 40, emap_final_size + 40, 0, 0)
    pdb.gimp_item_set_visible(emap_drawable, False)
	
    # Call Solid Noise Plug-in
    
    pdb.plug_in_solid_noise(image, emap_drawable, True, True, random_number, detail_level, horizontal_texture_size, vertical_texture_size)

    # Auto Stretch
	
    pdb.plug_in_autostretch_hsv( image, emap_drawable)
	
    # Gaussian Blur
	
    pdb.plug_in_gauss(image, emap_drawable, 5, 5, 0)

    # Set Specified Gradient Active Unless Active Gradient Is Selected
	
    if map_type < 2:
        pdb.gimp_context_set_gradient(bevel_gradient)
	
			
    # Environment Map Type: Topographical
    pdb.gimp_context_set_foreground(saved_fg_color)
    pdb.gimp_context_set_background(saved_bg_color)
	
    if map_type == 0 or map_type == 2:
        pdb.gimp_context_set_gradient(bevel_gradient)
        pdb.plug_in_gradmap(image, emap_drawable) # Map Gradient
        pdb.gimp_item_set_name(emap_drawable,"Spherical Topo") # Name emap


###############################################################################################################

	
    #  Environment Map Type: Cloud
    if paint_mode > 0 or map_type == 1 or map_type == 3:
        image_height = pdb.gimp_drawable_height(emap_drawable)
        image_width = pdb.gimp_drawable_width(emap_drawable)
		
    # Calculate Gradient Blend Start & End Points
	
        if blend_direction ==0: # Top-Bottom 
            xblend_start =  0
            yblend_start =  0
            xblend_end =  0
            yblend_end =  image_height
			
        elif blend_direction == 1: # Bottom-Top
            xblend_start =  0
            yblend_start =  image_height
            xblend_end =  0
            yblend_end =  0
			
        elif blend_direction == 2: # Left-Right
            xblend_start =  0
            yblend_start =  0
            xblend_end =  image_width
            yblend_end =  0
			
        elif blend_direction == 3: # Right-Left
            xblend_start =  image_width
            yblend_start =  0
            xblend_end = 0
            yblend_end =  0
	
        elif blend_direction == 4: # Diag-Top-Left
            xblend_start =  0
            yblend_start =  0
            xblend_end =  image_width
            yblend_end =  image_height
			
        elif blend_direction == 5: # Diag-Top-Right
            xblend_start =  image_width
            yblend_start =  0
            xblend_end =  0
            yblend_end =  image_height
			
        elif blend_direction == 6: # Diag-Bottom-Left
            xblend_start =  0
            yblend_start =  image_height
            xblend_end =  image_width
            yblend_end =  0
			
        elif blend_direction == 7: # Diag-Bottom-Right
            xblend_start =  image_width
            yblend_start =  image_height
            xblend_end =  0
            yblend_end =  0
			

    # Check Paint Mode
    if paint_mode == 0 and (map_type == 1 or map_type == 3):
        paint_mode = 4
        
        pdb.gimp_edit_blend(emap_drawable, BLEND_CUSTOM, paint_mode, blend_shape, 100, 0, 0,False, True, 3, 0.20, True, xblend_start, yblend_start, xblend_end, yblend_end)
			
	
    if map_type == 1 or map_type == 3:
        pdb.gimp_item_set_name(emap_drawable,"Spherical Cloud") # Name emap
					
######################################################################################################
    
    
    # Create Solid Color Spherical Map Using Current Foreground Color

			
    if map_type == 4:
	
        pdb.gimp_context_set_foreground(saved_fg_color)
        pdb.gimp_layer_add_alpha(emap_drawable)
        pdb.gimp_edit_clear(emap_drawable)
        pdb.gimp_drawable_fill(emap_drawable, 0) # fg fill
		
    # Gaussian Blur Final eMap
	
    if blur_radius>0:
	
        pdb.plug_in_gauss(image, emap_drawable, blur_radius, blur_radius, 0)
		
    # Make Environment Map Spherical
    if not map_type == 4:
        pdb.gimp_layer_add_alpha(emap_drawable)


    # replaced -1 values with None for python foo

    pdb.plug_in_map_object(image, emap_drawable, 1, 
                            0.5, 0.5, 2.0, 
                            0.5, 0.5, 0.0, 
                            1.0, 0.0, 0.0, 
                            0.0, 1.0, 0.0, 
                            0.0, 0.0, 0.0, 
                            0, white, 
                            -0.5, -0.5, 2.0, 
                            -1.0, -1.0, 1.0 , 
                            0.3, 1.0, 0.45, 
                            specular_highlight, 30.0, 
                            True, True, False, True, 
                            .44, 0.5, 0.5, 
                            0.5, 1.00, 
                            None, None, None, None, None, None, None, None)

    pdb.plug_in_autocrop_layer(image, emap_drawable)
	
	
    # Rotate Spherical Map For Lighting Direction 
	
    pdb.gimp_item_transform_rotate(emap_drawable,(((light_direction + 40) *(math.pi))) / 180, True, 0, 0)
	
    pdb.gimp_layer_set_offsets(emap_drawable, 0, 0)
	
    # End of emap render stuff - so return emap layer for ML BRL to use
	
    return emap_drawable
			
	
#### End of proc generate_emap

######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)

######################################################################################################################################
	
def proc_trim_character(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################
def do_brl(image, layer, trim_value, desaturate_option, slope_options, 
   flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, 
   diffuse_shiny, contour, blur_depth, outline_to_original, edge_smoothing, force_no_emap, emap_name, 
   map_type, keep_emap, paint_mode, blend_direction, blend_shape, light_direction, specular_highlight, 
   detail_level, blur_radius, 
   to_origin, artifact_radius, artifact_delta, help, ptab):
			
   pdb.gimp_image_set_active_layer(image,layer)
   brlLayer = pdb.gimp_layer_new_from_drawable(layer, image)
   pdb.gimp_image_insert_layer(image, brlLayer, None, -1)
   
   for layer in image.layers:
       pdb.gimp_image_set_active_layer(image,layer)
       pdb.gimp_item_set_visible(layer, 0)
       oldname = layer.name
       layer.name = str('#*#') + str(oldname)
   

   pdb.gimp_item_set_name(brlLayer, "|")
   pdb.gimp_item_set_visible(brlLayer, 1)

   do_stuff(image, pdb.gimp_image_get_layer_by_name(image, "|"), trim_value, desaturate_option, slope_options, 
   flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, 
   diffuse_shiny, contour, blur_depth, outline_to_original, edge_smoothing, force_no_emap, emap_name,  
   map_type, keep_emap, paint_mode, blend_direction, blend_shape, light_direction, specular_highlight, 
   detail_level, blur_radius, 
   0,0, to_origin, artifact_radius, artifact_delta, 0, 0, 0)
   
   pdb.gimp_item_set_name(pdb.gimp_image_get_layer_by_name(image, "|"), 'Layer BRL')
   layer=pdb.gimp_image_get_layer_by_name(image, "Layer BRL")
   for layer in image.layers:
      searchtermRegEx = re.compile(re.escape('#*#'))	
      layerName = searchtermRegEx.sub('', layer.name)
      layer.name = layerName 
   pdb.gimp_image_raise_item_to_top(image, pdb.gimp_image_get_layer_by_name(image, "Layer BRL"))   	  
   

   
hlp = " This is the Python version of GnuTux's Bevel Reflect Logo script.\n Preview for this plugin was added by MareroQ.\n\n	*** Select parameters, then turn on 'Preview' ***.\n\n Resizing method 'Preview':\n  1. Disable preview.\n  2. Drag the corner.\n  3. Turn on the preview again.\n\n Also turn off 'Preview' when You want to change\n some parameters.\n\n                 *** REQUIRED: MLBevelReflect2 ***"	\
+	"\n\n I wish You good use of this plugin:\n\n <b>                          Skinnyhouse.</b>"

register(
    "do_brl",
    "Applies ML Bevel Reflect2 plug-in\nto layer with transparency. Ver.1.10-P",
    "Applies ML Bevel Reflect 2 plug-in to Alpha to Logo in the styles 0f GnuTux",
    "Original code by GnuTux Python version by ",
    "Open source (BSD 3-clause license)",
    "2013 2022",
    "<Image>/Filters/Alpha to Logo/Bevel Reflect Logo...",
    "RGB*",
    [
    (PF_SPINNER, "pf_shine_char", "Trim characters (in pixels): ", 0,(0,10,1)),
    (PF_OPTION, "desaturate_option", "Desaturate emap result options\n(for applying to existing faces):", 0, ("None applied", "Desaturate Lightness", "Desaturate Luma", "Desaturate Average", "Desaturate Luminance", "Desaturate Value")),
    (PF_OPTION, "Slope_option", "Slope Options: ", 0, ("Flat Bevel", "Round 1 Bevel", "Round 2 Bevel")),
    (PF_SPINNER, "Flat_bevel_smoothing", "Flat Bevel - Smoothing: ", 32.0, (1.0, 32.0, 1.0)),
    (PF_SPINNER, "Flat_bevel_depth", "Flat Bevel - Depth: ", 10.0, (1.0, 10.0, 0.5)),
    (PF_SPINNER, "Round1_depth", "Round 1 Bevel - Depth: ", 8.0, (1.0, 8.0, 0.5)),
    (PF_SPINNER, "Round2_depth", "Round 2 Bevel - Depth: ", 25.0, (1.0, 25.0, 0.5)),
    (PF_SPINNER, "Diffuse_Shiny", "(Diffuse) 0<--->1 (Shiny):", 1.0, (0.0, 1.0, 0.1)),
    (PF_OPTION, "Contour_option", "Contour Options: ", 0, ("Linear", "Outline", "Outline (Inner Bevel)", "Outline (Outer Bevel)", "Step Bevel", "Ring (Narrow)", "Ring (Medium)", "Ring (Wide)", "Ring (Extra Wide)", "Ring (Thick)", "Double Ring", "Double Ring (Narrow)", "Double Ring (Wide)")),
    (PF_SPINNER, "Contour_depth", "Contour Depth: \n(Use bigger values\nfor Outline Contour)", 0, (0, 20, 1)),
    (PF_TOGGLE, "pf_outline", "Apply outlines to original\ncharacters: ", 0, [("Outlines and characters"),("Outlines only") ]),
    (PF_SPINNER, "Edge_smoothing", "Edge Smoothing Amount: ", 4, (0, 4, 1)),
    (PF_TOGGLE, "pf_force_no_emap", "Force emap image from file?", 1, [("Load Emaps"),("Created New by Options 2") ]),
    (PF_FILE, "pf_emap_name", "Spherical Emap (png/xcf): ", ""),
	
    (PF_OPTION, "Environment_Map_Type", "Environment Map Type: ", 0, ("Spherical Topographical", "Spherical Cloud", "Spherical Topo (Active Gradient)", "Spherical Cloud (Active Gradient)", "Spherical Solid (Active Color)"),{'newLine':'1','tab':1}),
    (PF_TOGGLE, "keep_emap", "Keep newly created emap\n(as a layer): ", 1, [("Keep it"),("Remove")],{'newLine':'1','tab':1}),
    (PF_OPTION, "Blending_Mode", "Blending Mode: ", 0, ("Normal", "Dissolve", "Behind", "Multiply", "Screen", "Overlay", "Difference", "Addition", "Subtract", "Darken", "Lighten", "Hue", "Saturation", "Color", "Value", "Divide", "Dodge", "Burn", "Hard Light", "Soft Light", "Grain Extract", "Grain Merge"),{'newLine':'1','tab':1}),
    (PF_OPTION, "Blend_Direction", "Blend Direction: ", 0, ("Top-Bottom", "Bottom-Top", "Left-Right", "Right-Left", "Diag-Top-Left", "Diag-Top-Right", "Diag-Bottom-Left", "Diag-Bottom-Right"),{'newLine':'1','tab':1}),
    (PF_OPTION, "Blend_Shape", "Blend Shape: ", 0, ("Linear", "Bilinear", "Radial", "Square", "Conical-Symmetric", "Conical-Asymmetric", "Shapeburst-Angular", "Shapburst-Spherical", "Shapeburst-Dimpled", "Spiral-CW", "Spiral-CCW"),{'newLine':'1','tab':1}),
    (PF_SPINNER, "Light_Direction", "Light Direction (Degrees): ", 315, (0, 360, 45),{'newLine':'1','tab':1}),
    (PF_SPINNER, "Emap_Specular_Highlights", "Emap Specular Highlights: ", 0.65, (0.10, 0.65, 0.05),{'newLine':'1','tab':1}),
    (PF_SPINNER, "Detail_Level", "Detail Level: ", 3, (0, 3, 1),{'newLine':'1','tab':1}),
    (PF_SPINNER, "EMap_Final_Blur", "EMap Final Blur: ", 0, (0, 20, 1),{'newLine':'1','tab':1}),
    #(PF_OPTION, "Char_option", "Emap to character Options: ", 0, ("Single emap", "Random sequence emap", "Random emap", "Character grouping emaps", "Vowel/consonant grouping emaps"),{'newLine':'1','tab':1}),
    #(PF_TOGGLE, "pf_save_emap", "Open any generated emaps\nin as new image: ", 0, [("Do not save images"),("Save images") ],{'newLine':'1','tab':1}),
    (PF_TOGGLE, "pf_layer_origins", "Position active layers: ", 0, [("Layer to origin Image"),("Leave layer as they are"), ],{'newLine':'1','tab':1}),
    (PF_SPINNER, "pf_blur_radius", "Artifact blur radius: ", 1,(1,100,1),{'newLine':'1','tab':1}),
    (PF_SPINNER, "pf_blur_delta", "Artifact blur delta amount: ", 10,(0,255,1),{'newLine':'1','tab':1}),
    #(PF_TOGGLE, "Preview_option", "Preview single character\n(to a new image): ", 0, [("Preview off"),("Preview on") ],{'newLine':'1','tab':1}),
    #(PF_STRING, "preview_string", "Preview character: ", ("A"), 0,{'newLine':'1','tab':1}),
    (PF_TITLE  , "help"  , hlp ,(10,10), None           ,{'tab':2}),	
    (PF_TAB    , "ptab"    , "tabs", 0, (_('Options 1'),_('Options 2'),_('Help')),{'newLine':'1','tab':2} ),
	
    ],
    [],
    do_brl)

main()
